/*
    tstore - an archive format for archiving tar archives
    Copyright (C) 2004-2007  Kasper Dupont

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    with permission from the author any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    $Id: tstore.h,v 1.21 2008/04/13 09:43:40 kasperd Exp $
*/

#ifndef _TSTORE_H
#define _TSTORE_H

#define _GNU_SOURCE
#define _LARGEFILE_SOURCE  
#define _LARGEFILE64_SOURCE  
#define _FILE_OFFSET_BITS 64

#define _XOPEN_SOURCE 500

#include <errno.h>
#include <inttypes.h> // Used to be stdint.h, but this is more portable
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <stdio.h>
#include <ctype.h>
#include <assert.h>

#include <unistd.h>
#include <fcntl.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/time.h>

#include "tsdefs.h"

struct store {
  uint8_t header_buf[512];
  int fd,locked;
  uint64_t store_size;
  char *name,*h32name;
};

enum lock { DONT_LOCK, CHECK_LOCK };

extern uint8_t hash_key[16];
void hash_buf(uint8_t *buf, int size);
ssize_t read_and_hash(int fd, uint8_t *buf);

void close_store(struct store *s);
struct store *open_store(const char *name, int flags, enum lock lock);
int lock_store(struct store *s);
int find_file(struct store *s,
	      const char *name,
	      off_t *header_offset,
	      off_t *tree_offset,
	      off_t *size,
	      struct timespec *mtime,
	      uint8_t *header_buf);
ssize_t myread(int fd, void *buf, size_t count);
int filelevels(uint64_t size);
int seqblock(uint8_t *buf);
void get_block(struct store*, uint64_t, uint8_t*);

uint64_t get_nbyteint(uint8_t *p, int s);
void put_nbyteint(uint8_t *p, int s, uint64_t n);

typedef uint32_t wtree_key_t;
typedef struct wtree *wtree_t;
wtree_t insert(wtree_t *tp, wtree_t newnode);
wtree_t find(wtree_t t, uint8_t *elm);
void discard(wtree_t t);
uint64_t *locationp(wtree_t t);
uint8_t *datap(wtree_t t);
wtree_t allocnode();
wtree_t find_hash(wtree_t t, wtree_key_t h);
void discard_tree();
extern wtree_t hsec_tree;

int confirm();
void cancel();
void errstop(const char *s);

static inline void *myalloc(size_t size)
{
  void *r=calloc(1,size);
  if (!r) errstop("malloc");
  return r;
}
#define NEW(v) v=myalloc(sizeof(*(v)))

#endif
